import { getScreenSelector, ScreenWidthMaxSmall, getGlobalClassNames, IconFontSizes, HighContrastSelector, getFocusStyle } from '@uifabric/styling';
import _defineProperty from '@babel/runtime/helpers/defineProperty';
import { MessageBarType } from './MessageBar.types.js';

var _backgroundColor, _highContrastBackgrou, _iconColor;

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }
var GlobalClassNames = {
  root: 'ms-MessageBar',
  error: 'ms-MessageBar--error',
  blocked: 'ms-MessageBar--blocked',
  severeWarning: 'ms-MessageBar--severeWarning',
  success: 'ms-MessageBar--success',
  warning: 'ms-MessageBar--warning',
  multiline: 'ms-MessageBar-multiline',
  singleline: 'ms-MessageBar-singleline',
  dismissalSingleLine: 'ms-MessageBar-dismissalSingleLine',
  expandingSingleLine: 'ms-MessageBar-expandingSingleLine',
  content: 'ms-MessageBar-content',
  iconContainer: 'ms-MessageBar-icon',
  text: 'ms-MessageBar-text',
  innerText: 'ms-MessageBar-innerText',
  dismissSingleLine: 'ms-MessageBar-dismissSingleLine',
  expandSingleLine: 'ms-MessageBar-expandSingleLine',
  dismissal: 'ms-MessageBar-dismissal',
  expand: 'ms-MessageBar-expand',
  actions: 'ms-MessageBar-actions',
  actionsSingleline: 'ms-MessageBar-actionsSingleLine'
};
var backgroundColor = (_backgroundColor = {}, _defineProperty(_backgroundColor, MessageBarType.error, 'errorBackground'), _defineProperty(_backgroundColor, MessageBarType.blocked, 'errorBackground'), _defineProperty(_backgroundColor, MessageBarType.success, 'successBackground'), _defineProperty(_backgroundColor, MessageBarType.warning, 'warningBackground'), _defineProperty(_backgroundColor, MessageBarType.severeWarning, 'severeWarningBackground'), _defineProperty(_backgroundColor, MessageBarType.info, 'infoBackground'), _backgroundColor);
var highContrastBackgroundColor = (_highContrastBackgrou = {}, _defineProperty(_highContrastBackgrou, MessageBarType.error, 'rgba(255, 0, 0, 0.3)'), _defineProperty(_highContrastBackgrou, MessageBarType.blocked, 'rgba(255, 0, 0, 0.3)'), _defineProperty(_highContrastBackgrou, MessageBarType.success, 'rgba(48, 241, 73, 0.3)'), _defineProperty(_highContrastBackgrou, MessageBarType.warning, 'rgba(255, 254, 57, 0.3)'), _defineProperty(_highContrastBackgrou, MessageBarType.severeWarning, 'rgba(255, 0, 0, 0.3)'), _defineProperty(_highContrastBackgrou, MessageBarType.info, 'Window'), _highContrastBackgrou);
var iconColor = (_iconColor = {}, _defineProperty(_iconColor, MessageBarType.error, 'errorIcon'), _defineProperty(_iconColor, MessageBarType.blocked, 'errorIcon'), _defineProperty(_iconColor, MessageBarType.success, 'successIcon'), _defineProperty(_iconColor, MessageBarType.warning, 'warningIcon'), _defineProperty(_iconColor, MessageBarType.severeWarning, 'severeWarningIcon'), _defineProperty(_iconColor, MessageBarType.info, 'infoIcon'), _iconColor);
var getStyles = function getStyles(props) {
  var theme = props.theme,
      className = props.className,
      onDismiss = props.onDismiss,
      truncated = props.truncated,
      isMultiline = props.isMultiline,
      expandSingleLine = props.expandSingleLine,
      _props$messageBarType = props.messageBarType,
      messageBarType = _props$messageBarType === void 0 ? MessageBarType.info : _props$messageBarType;
  var semanticColors = theme.semanticColors,
      fonts = theme.fonts;
  var SmallScreenSelector = getScreenSelector(0, ScreenWidthMaxSmall);
  var classNames = getGlobalClassNames(GlobalClassNames, theme);
  var dismissalAndExpandIconStyle = {
    fontSize: IconFontSizes.xSmall,
    height: 10,
    lineHeight: '10px',
    color: semanticColors.messageText,
    selectors: _defineProperty({}, HighContrastSelector, {
      MsHighContrastAdjust: 'none',
      color: 'WindowText'
    })
  };
  var dismissalAndExpandStyle = [getFocusStyle(theme, {
    inset: 1,
    highContrastStyle: {
      outlineOffset: '-6px',
      outline: '1px solid Highlight'
    },
    borderColor: 'transparent'
  }), {
    flexShrink: 0,
    width: 32,
    height: 32,
    padding: '8px 12px',
    selectors: {
      '& .ms-Button-icon': dismissalAndExpandIconStyle,
      ':hover': {
        backgroundColor: 'transparent'
      },
      ':active': {
        backgroundColor: 'transparent'
      }
    }
  }];
  return {
    root: [classNames.root, fonts.medium, messageBarType === MessageBarType.error && classNames.error, messageBarType === MessageBarType.blocked && classNames.blocked, messageBarType === MessageBarType.severeWarning && classNames.severeWarning, messageBarType === MessageBarType.success && classNames.success, messageBarType === MessageBarType.warning && classNames.warning, isMultiline ? classNames.multiline : classNames.singleline, !isMultiline && onDismiss && classNames.dismissalSingleLine, !isMultiline && truncated && classNames.expandingSingleLine, {
      background: semanticColors[backgroundColor[messageBarType]],
      color: semanticColors.messageText,
      minHeight: 32,
      width: '100%',
      display: 'flex',
      wordBreak: 'break-word',
      selectors: _defineProperty({
        '.ms-Link': {
          color: semanticColors.messageLink,
          selectors: {
            ':hover': {
              color: semanticColors.messageLinkHovered
            }
          }
        }
      }, HighContrastSelector, {
        MsHighContrastAdjust: 'none',
        background: highContrastBackgroundColor[messageBarType],
        border: '1px solid WindowText',
        color: 'WindowText'
      })
    }, isMultiline && {
      flexDirection: 'column'
    }, className],
    content: [classNames.content, {
      display: 'flex',
      width: '100%',
      lineHeight: 'normal'
    }],
    iconContainer: [classNames.iconContainer, {
      fontSize: IconFontSizes.medium,
      minWidth: 16,
      minHeight: 16,
      display: 'flex',
      flexShrink: 0,
      margin: '8px 0 8px 12px'
    }],
    icon: {
      color: semanticColors[iconColor[messageBarType]],
      selectors: _defineProperty({}, HighContrastSelector, {
        MsHighContrastAdjust: 'none',
        color: 'WindowText'
      })
    },
    text: [classNames.text, _objectSpread(_objectSpread({
      minWidth: 0,
      display: 'flex',
      flexGrow: 1,
      margin: 8
    }, fonts.small), {}, {
      selectors: _defineProperty({}, HighContrastSelector, {
        MsHighContrastAdjust: 'none'
      })
    }), !onDismiss && {
      marginRight: 12
    }],
    innerText: [classNames.innerText, {
      lineHeight: 16,
      selectors: {
        '& span a': {
          paddingLeft: 4
        }
      }
    }, truncated && {
      overflow: 'visible',
      whiteSpace: 'pre-wrap'
    }, !isMultiline && {
      // In high contrast this causes the top and bottom of links' focus outline to be clipped
      // (not sure of a good way around that while still maintaining text clipping)
      overflow: 'hidden',
      textOverflow: 'ellipsis',
      whiteSpace: 'nowrap'
    }, !isMultiline && !truncated && {
      selectors: _defineProperty({}, SmallScreenSelector, {
        overflow: 'visible',
        whiteSpace: 'pre-wrap'
      })
    }, expandSingleLine && {
      overflow: 'visible',
      whiteSpace: 'pre-wrap'
    }],
    dismissSingleLine: classNames.dismissSingleLine,
    expandSingleLine: classNames.expandSingleLine,
    dismissal: [classNames.dismissal, dismissalAndExpandStyle],
    expand: [classNames.expand, dismissalAndExpandStyle],
    actions: [isMultiline ? classNames.actions : classNames.actionsSingleline, {
      display: 'flex',
      flexGrow: 0,
      flexShrink: 0,
      flexBasis: 'auto',
      flexDirection: 'row-reverse',
      alignItems: 'center',
      margin: '0 12px 0 8px',
      selectors: {
        '& button:nth-child(n+2)': {
          marginLeft: 8
        }
      }
    }, isMultiline && {
      marginBottom: 8
    }, onDismiss && !isMultiline && {
      marginRight: 0
    }]
  };
};

export { getStyles };
