import { addElementAtIndex } from '@uifabric-vue/utilities';
import _toConsumableArray from '@babel/runtime/helpers/toConsumableArray';
import { DATAKTP_TARGET, KTP_SEPARATOR, KTP_PREFIX } from './KeytipConstants.js';

/**
 * Converts a whole set of KeySequences into one keytip ID, which will be the ID for the last keytip sequence specified
 * keySequences should not include the initial keytip 'start' sequence.
 *
 * @param keySequences - Full path of IKeySequences for one keytip.
 * @returns {string} String to use for the keytip ID.
 */

function sequencesToID(keySequences) {
  return keySequences.reduce(function (prevValue, keySequence) {
    return prevValue + KTP_SEPARATOR + keySequence.split('').join(KTP_SEPARATOR);
  }, KTP_PREFIX);
}
/**
 * Merges an overflow sequence with a key sequence.
 *
 * @param keySequences - Full sequence for one keytip.
 * @param overflowKeySequences - Full overflow keytip sequence.
 * @returns {string[]} Sequence that will be used by the keytip when in the overflow.
 */

function mergeOverflows(keySequences, overflowKeySequences) {
  var overflowSequenceLen = overflowKeySequences.length;

  var overflowSequence = _toConsumableArray(overflowKeySequences).pop();

  var newKeySequences = _toConsumableArray(keySequences);

  return addElementAtIndex(newKeySequences, overflowSequenceLen - 1, overflowSequence);
}
/**
 * Constructs the data-ktp-target attribute selector from a full key sequence.
 *
 * @param keySequences - Full string[] for a Keytip.
 * @returns {string} String selector to use to query for the keytip target.
 */

function ktpTargetFromSequences(keySequences) {
  return '[' + DATAKTP_TARGET + '="' + sequencesToID(keySequences) + '"]';
}

export { ktpTargetFromSequences, mergeOverflows, sequencesToID };
